import { createElement, Fragment, useMemo, useState } from "react";
import { isObservable } from "rxjs";
import { takeUntil } from "rxjs/operators";
import { useDestroyObservable } from "./shared";
// TODO: add better TS support
/**
 * <$> fragment will subscribe to it's Observable children and display
 * it's emissions along with regular children
 *
 * e.g.
 *
 * ```jsx
 * function App(){
 *   return <$>{ timer(0, 1000) }</$>    // 0, 1, 2, 3, ...
 * }
 * ```
 */
export function $(props) {
    const children = props === null || props === void 0 ? void 0 : props.children;
    // store children
    const [cn, setCn] = useState(null);
    const [error, setError] = useState(null);
    const destroy$ = useDestroyObservable();
    useMemo(() => {
        // array of children, one of em might be observable
        if (Array.isArray(children)) {
            setCn(children.map(c => createElement($, null, c)));
            return;
        }
        // child is a single observable
        if (isObservable(children)) {
            children
                .pipe(takeUntil(destroy$))
                .subscribe({
                next: setCn,
                error(error) {
                    setError({ error });
                }
            });
            return;
        }
        // child is a regular child, like you and me
        setCn(children);
    }, [children]);
    // raise an error if Observable failed
    if (error) {
        throw error.error;
    }
    // NOTE: array children are rendered inside <></> fragment
    //       while regular children are rendered directly as children of <$>,
    //       without additional <>
    //       this might be an unwanted behavior difference
    // TODO: research for real-life effects of this difference
    return Array.isArray(cn)
        ? createElement(Fragment, null, ...cn)
        : cn;
}
